%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  DemoRF3D (ver. 1.00, 22 September 2014)
%
%  DemoRF3D demonstrates the algorithm for joint attenuation of random 
%  (RND) and Fixed-Pattern (FPN) noise described in the paper:
%
% [1] M. Maggioni, E. Snchez-Monge, A. Foi, "Joint removal of random and
%     fixed-pattern noise through spatiotemporal video filtering", IEEE 
%     Transactions on Image Processing, vol.23, no.10, pp. 4282-4296, Oct. 2014
%     http://doi.org/10.1109/TIP.2014.2345261
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Copyright (c) 2011-2014    All rights reserved.
% This work should only be used for nonprofit purposes.
% Any unauthorized use of the provided software and files for industrial
% or profit-oriented activities is expressively prohibited.
% By using these files, you implicitly agree to all the terms of the
% TUT limited license, as specified in the document
% LEGAL_NOTICE.txt included in this package, and online at
% http://www.cs.tut.fi/~foi/GCF-BM3D/legal_notice.html
% Please read the TUT limited license before you proceed with using these
% files.
%
% AUTHORS:
%     Enrique Snchez-Monge  < esm _at_ noiselessimaging.com >
%     Matteo Maggioni  < matteo.maggioni _at_ tut.fi >
%     Alessandro Foi  < alessandro.foi _at_ tut.fi >
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%  MODIFIABLE DEMO PARAMETERS
%%%%

% Choose experiment type
experiment_type = 0;       % Available options:
                           %  0 --> denoising of real noisy video acquired by a 
                           %        FLIR Tau 320 microbolometer camera
                           %  1 --> denoising of video corrupted by synthetic noise

% Path to video file (sequences available at http://www.cs.tut.fi/~foi/GCF-BM3D)
file_name = '';            % If empty, a prompt dialog will appear. The file
                           % type allowed to be selected depends on experiment_type.

% Noise parameters used to generate the synthetic random and fixed-pattern
% noise added to the noise-free videos used when experiment_type = 1.
% These parameters apply IF AND ONLY IF experiment_type = 1.
% Real noisy videos do not require generation of additional synthetic noise.
sigma_rnd = 15;            % Scaling factor of random noise (must be >=0).
sigma_fpn = 15;            % Scaling factor of fixed-pattern noise (FPN) (must be >=0).
FPN_drift = 0.0;           % FPN drift (0<=FPN_drift<=1).
                           % If equal to 0, the FPN is static (i.e. no drift).
                           % If 0 < FPN_drift < 1, the FPN drifts at each frame.
                           % If FPN_drift = 1, the FPN component changes completely
                           % at every frame, thus degenerating into another RND component.
                           % FPN_drift =  0  was used for Table I in [1].
                           % FPN_drift =  0.01  was used for Table II in [1].

% RF3D parameters
profile           = 'np';  % Complexity profile:
                           %  'np' --> Normal Profile (balanced quality, default)
                           %  'lc' --> Low Complexity Profile (fast, lower quality)
transform_2D_name = 'dct'; % 2D block transform used in RF3D (it also defines the PSDs)
                           % RF3D accepts 'dct', 'dst', 'hadamard', as well
                           % as everything listed by 'help wfilters'.
                           % However, the same transform operator defines
                           % the PSDs of the noise. We only provide the
                           % PSDs for 'dct', 'bior1.5', or 'haar'.
do_wiener         = true;  % Wiener Filtering (true or false).
erf3d             = true;  % Enhanced FPN suppression (true or false).
print_to_screen   = true;  % Verbose mode (true or false).




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%  RF3D FILTERING (NOTHING SHOULD BE CHANGED BELOW THIS POINT)
%%%%

% In both cases, the PSDs should be defined to the same 2D transform used 
% in RF3D (i.e., transform_2D_name). In this package, we provide the PSDs  
% for 'dct', 'bior1.5', and 'haar' transform.
if experiment_type==0
    % Loading real data acquired by FLIR Tau 320 camera and the corresponding PSDs
    [z, psd_rnd, psd_fpn] = LoadRealData(file_name, transform_2D_name);
    
elseif experiment_type==1
    % Loading noise-free and noisy synthetic data and the corresponding PSDs
    [y, z, psd_rnd, psd_fpn] = LoadSyntheticData(file_name, transform_2D_name, sigma_rnd, sigma_fpn, FPN_drift);
    
else
    % Invalid experiment_type option
    error('Invalid "experiment_type" option. Plese, select either 0 or 1.');
end

% RF3D filtering execution (refer to RF3D.m for additional information).
% The PSDs of the noise are assumed to be known, whereas the scaling
% factors are always automatically estimated from the noisy data.
timeFirst = now;
y_est = RF3D(z, psd_rnd, psd_fpn, transform_2D_name, erf3d, do_wiener, profile, print_to_screen);
estimate_elapsed_time = 86400*(now-timeFirst);

% Showing objective (PSNR) and subjective results
fprintf('Filtering completed: %.1f sec (%.1f fps)', ...
    estimate_elapsed_time, size(z,3)/estimate_elapsed_time);
if experiment_type==1 && exist('y','var') && ~isempty(y)
    PSNR = 10*log10(255^2/mean((y(:)-y_est(:)).^2)); % the assumed signal peak is 255
    fprintf(', PSNR %.2f dB. \n', PSNR);
else
    fprintf('.\n');
end

implay(([z y_est]-min(y_est(:)))/(max(y_est(:))-min(y_est(:))))

